package com.chinacreator.ecd.common.service;

import java.util.List;

import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.chinacreator.ecd.common.BusinessException;
import com.chinacreator.ecd.common.ErrorInfo;
import com.chinacreator.ecd.common.entity.InterfaceUserEntity;
import com.chinacreator.ecd.common.mapper.InterfaceUserMapper;

@Service
public class InterfaceUserService {
	
private final static Logger logger = LoggerFactory.getLogger(InterfaceUserService.class);
	
	@Autowired
	private InterfaceUserMapper interfaceUserMapper;
	
	@Cacheable(cacheNames="interfaceUsers")
	public List<InterfaceUserEntity> findByUsername(String username, String interfacename) {
		return interfaceUserMapper.findByUsername(username, interfacename);
	}
	
	/**
	 * 判断该IP是否有权限调用接口
	 * 
	 * @param requestIp
	 * @param channel
	 */
	public void validateIp(String username, String interfacename, String requestIp) {
		
		try {
			List<InterfaceUserEntity> list = findByUsername(username, interfacename);
			
			if (list == null || list.size() == 0) {
	            throw new BusinessException("10011", "未查询到接口用户信息，请确定是否配置（" + username + ", " + interfacename + "）") ;
	        }
	        
	        for (InterfaceUserEntity entity : list) {
	        	
	        	logger.info("username={}, interfacename={}, allowid={}", username, interfacename, entity.getAllowip());
	        	
	        	// IP地址未配置，看下一条
	        	if (StringUtils.isEmpty(entity.getAllowip())) {
	        		continue;
	        	}
	        	
	        	// IP地址配置的是*号，允许所有IP访问
	        	if ("*".equals(entity.getAllowip())) {
	                return;
	            }
	        	
	        	String[] ips = entity.getAllowip().toString().replaceAll(",", "|").split("\\|");
	            for (String ip : ips) {
	                if (!StringUtils.isEmpty(ip) && requestIp.contains(ip)) {
	                    return;
	                }
	                
	                //如果IP地址配置有*号，访问的IP包含*号前面的部分
	                if (ip.indexOf("*") >= 0 && ip.startsWith(ip.split("[*]")[0])) {
	                	return;
	                }
	            }
	        }
	        
	        
			throw new BusinessException(ErrorInfo.code1002, ErrorInfo.errorDesc.get(ErrorInfo.code1002) + "[" + requestIp + "]!");
			
		} catch (BusinessException e) {
			
			logger.error("IP鉴权失败：username={}, interfacename={}, requestIp={}", username, interfacename, requestIp);
			throw new BusinessException(e.getResultcode(), e.getErrorinfo()) ;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("IP鉴权失败：requestIp={}, username={}, interfacename={}, reason={}", requestIp, username, interfacename, e.getMessage());
			throw new BusinessException(ErrorInfo.code1002, ErrorInfo.errorDesc.get(ErrorInfo.code1002) + "[" + requestIp + "]!");
		}
	}
	
	/*
	 * timestamp 有效期校验
	 * @param timestamp
	 */
	public void validateTimestamp(String username, String interfacename, String timestamp) {
		
		try {
			List<InterfaceUserEntity> list = findByUsername(username, interfacename);
			
			if (list == null || list.size() == 0) {
	            throw new BusinessException("10011", "未查询到接口用户信息，请确定是否配置'" + username + ", " + interfacename + "'") ;
	        }
	
			for (InterfaceUserEntity entity : list) {
				if (StringUtils.isEmpty(entity.getDuration())) {
					continue;
				}
				
				Long duration = Long.valueOf(entity.getDuration());
				duration = duration * 60; // 分钟转换成秒
				
				if (System.currentTimeMillis()/1000 - Long.valueOf(timestamp) <= duration) {
					return;
				}
			}
			
			throw new BusinessException(ErrorInfo.code1325, ErrorInfo.errorDesc.get(ErrorInfo.code1325));
		} catch (BusinessException e) {
			
			logger.error("timestamp鉴权失败：username={}, interfacename={}, timestamp={}", username, interfacename, timestamp);
			throw new BusinessException(e.getResultcode(), e.getErrorinfo()) ;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("timestamp鉴权失败：requestIp={}, username={}, interfacename={}, reason={}", timestamp, username, interfacename, e.getMessage());
			throw new BusinessException(ErrorInfo.code1002, ErrorInfo.errorDesc.get(ErrorInfo.code1002) + "[" + timestamp + "]!");
		}
	}
	
	/*
	 * signature 校验
	 * @param timestamp
	 */
	public void validateSignature(String username, String interfacename, String params, String timestamp, String signature) {
		
		try {
			List<InterfaceUserEntity> list = findByUsername(username, interfacename);
			
			if (list == null || list.size() == 0) {
	            throw new BusinessException("10011", "未查询到接口用户信息，请确定是否配置'" + username + ", " + interfacename + "'") ;
	        }
			
			for (InterfaceUserEntity entity : list) {
				
				String newSignature = DigestUtils.md5Hex(params + timestamp + entity.getPassword());
				if (newSignature.equals(signature)) {
					return;
				}
			}
			
			throw new BusinessException(ErrorInfo.code1326, ErrorInfo.errorDesc.get(ErrorInfo.code1326));
			
		} catch (BusinessException e) {
			
			logger.error("signature鉴权失败：username={}, interfacename={}, params={}, timestamp={}, signature={}", username, interfacename, params, timestamp, signature);
			throw new BusinessException(e.getResultcode(), e.getErrorinfo()) ;
			
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("signature鉴权失败：username={}, interfacename={}, params={}, timestamp={}, signature={}, reason={}", 
					username, interfacename, params, timestamp, signature, e.getMessage());
			throw new BusinessException(ErrorInfo.code1002, ErrorInfo.errorDesc.get(ErrorInfo.code1002) + "[" + timestamp + "]!");
		}
	}
	
	public static void main(String[] args) {
		
		System.out.println("127.0.3.1,127.0.3.2");
	}
}
